@@ -17,6 +17,6 @@
 #include "common.h"
 #include "mmsystem.h"
 void cmenu_browse_all(GtkWidget* w, gpointer data);
-void create_pbar(char*);
+void create_pbar(CHAR*);
 void destroy_pbar();
 #endif
