@@ -11,6 +11,7 @@
 #define TEMPLATE "template1"
 #ifndef __MORGOTHDBMA_PGXEXPLORER_H__
 #define __MORGOTHDBMA_PGXEXPLORER_H__
+#include "common.h"
 struct main_components
 {
  GtkWidget* table;
@@ -37,12 +38,10 @@ struct main_components
  GtkWidget* button_delete;
  GtkWidget*** table_entry;
  GtkWidget* last_selected;
- int ntables,nviews,ntriggers,cols,rows,acols,arows,rrow,rcol,last_row;
+ INT ntables,nviews,ntriggers,cols,rows,acols,arows,rrow,rcol,last_row;
  /* cols HOW MUCH ON WINDOW, acols ALL COLUMNS, RCOL, ROTATE_COL, OFFSET COL ON WINDOW */
- char selected[STDSTRING];
- char seltype;
- char*** data;
+ CHAR selected[STDSTRING];
+ CHAR seltype;
+ CHAR*** data;
 };
-
 #endif
-
