@@ -12,15 +12,13 @@
 #include "signals.h"
 /*FIXME dodac obsluge debug, error ==> DlgWindow*/
 /* THERE WOULDN'T BE ANY DEBUG LOG TO FILE ETC. */
-int ping_conn;
-extern int timeout;
+INT ping_conn;
+extern INT timeout;
 
-
-
-int connAlive(PGconn* c)
+INT connAlive(PGconn* c)
 {
  PGresult* r;
- int stat;
+ INT stat;
  debug("%s:%d connAlive(%p)", __FILE__,__LINE__,c);
  ping_conn=1;
  alarm_clock(timeout);
@@ -50,13 +48,13 @@ void disconnect_db(PGconn** c)
 }
 
 
-PGconn* connect_db(char* arg, char* pass)
+PGconn* connect_db(CHAR* arg, CHAR* pass)
 {
  PGconn* conn;
- char* strarg;
+ CHAR* strarg;
  conn=0;
  debug("%s:%d connect_db(%s,***)", __FILE__,__LINE__,arg?arg:"null");
- strarg = (char*)malloc(strlen(arg)+64+(pass?strlen(pass):1));
+ strarg = (CHAR*)malloc(strlen(arg)+64+(pass?strlen(pass):1));
  debug("\nconnect to database %s\n", arg);
  if (!pass) sprintf(strarg,"dbname=template1");
  else	     sprintf(strarg,"dbname=template1 password='%s'", pass);
@@ -84,13 +82,13 @@ PGconn* connect_db(char* arg, char* pass)
 }
 
 
-PGconn* gtk_connect_db(char* arg, char* pass, char* errstr)
+PGconn* gtk_connect_db(CHAR* arg, CHAR* pass, CHAR* errstr)
 {
  PGconn* conn;
- char* strarg;
+ CHAR* strarg;
  conn=0;
  debug("%s:%d gtk_connect_db(%s,***)", __FILE__,__LINE__,arg?arg:"null");
- strarg = (char*)malloc(strlen(arg)+64+(pass?strlen(pass):1));
+ strarg = (CHAR*)malloc(strlen(arg)+64+(pass?strlen(pass):1));
  if (!pass) sprintf(strarg,"dbname=template1");
  else       sprintf(strarg,"dbname=template1 password='%s'", pass);
  alarm_clock(timeout);
@@ -124,10 +122,10 @@ void clear_result(PGresult** r)
 }
 
 
-PGresult* execute_silent_query(PGconn* c,  char* query)
+PGresult* execute_silent_query(PGconn* c,  CHAR* query)
 {
  PGresult* r;
- int stat;
+ INT stat;
  debug("%s:%d execute_silent_query(%s)", __FILE__,__LINE__,query);
  debug("SQLquery: \"%s\"\n", query);
  if (!connAlive(c)) { debug("connection dead!\n"); return NULL; }
@@ -154,11 +152,11 @@ PGresult* execute_silent_query(PGconn* c,  char* query)
 }
 
 
-char*** p3c_execute_printf(PGconn* c,int* row, int* col, char* errstr, int want_meta, char* fmt,...)
+CHAR*** p3c_execute_printf(PGconn* c,INT* row, INT* col, CHAR* errstr, INT want_meta, CHAR* fmt,...)
 {
  va_list ap;
- char*** ret;
- char msg[MAX_QUERY_LENGTH*2];
+ CHAR*** ret;
+ CHAR msg[MAX_QUERY_LENGTH*2];
  debug("%s:%d p3c_execute_printf(%s, %d)", __FILE__,__LINE__,fmt,want_meta);
  if (!c) { error("no connection"); return NULL; }
  va_start(ap, fmt);
@@ -168,11 +166,12 @@ char*** p3c_execute_printf(PGconn* c,int* row, int* col, char* errstr, int want_
  return ret;
 }
 
-char*** dmp3c_execute_printf(PGconn* c,int* row, int* col, char* errstr, int want_meta, char* file, int line, char* fmt,...)
+
+CHAR*** dmp3c_execute_printf(PGconn* c,INT* row, INT* col, CHAR* errstr, INT want_meta, CHAR* file, INT line, CHAR* fmt,...)
 {
  va_list ap;
- char*** ret;
- char msg[MAX_QUERY_LENGTH*2];
+ CHAR*** ret;
+ CHAR msg[MAX_QUERY_LENGTH*2];
  debug("%s:%d dmp3c_execute_printf(%s,%d) called from %s:%d", __FILE__,__LINE__,fmt,want_meta,file,line);
  if (!c) { error("no connection"); return NULL; }
  va_start(ap, fmt);
@@ -182,11 +181,12 @@ char*** dmp3c_execute_printf(PGconn* c,int* row, int* col, char* errstr, int wan
  return ret;
 }
 
-char*** dmem_p3c_execute_printf(struct dmem_list** mem, PGconn* c,int* row, int* col, char* errstr, int want_meta, char* file, int line, char* fmt,...)
+
+CHAR*** dmem_p3c_execute_printf(struct dmem_list** mem, PGconn* c,INT* row, INT* col, CHAR* errstr, INT want_meta, CHAR* file, INT line, CHAR* fmt,...)
 {
  va_list ap;
- char*** ret;
- char msg[MAX_QUERY_LENGTH*2];
+ CHAR*** ret;
+ CHAR msg[MAX_QUERY_LENGTH*2];
  debug("%s:%d dmem_p3c_execute_printf(%s,%d) called from %s:%d", __FILE__,__LINE__,fmt,want_meta,file,line);
  if (!c) { error("no connection"); return NULL; }
  va_start(ap, fmt);
@@ -196,11 +196,12 @@ char*** dmem_p3c_execute_printf(struct dmem_list** mem, PGconn* c,int* row, int*
  return ret;
 }
 
-char*** mp3c_execute_printf(PGconn* c,int* row, int* col, char* errstr, int want_meta, char* fmt,...)
+
+CHAR*** mp3c_execute_printf(PGconn* c,INT* row, INT* col, CHAR* errstr, INT want_meta, CHAR* fmt,...)
 {
  va_list ap;
- char*** ret;
- char msg[MAX_QUERY_LENGTH*2];
+ CHAR*** ret;
+ CHAR msg[MAX_QUERY_LENGTH*2];
  debug("%s:%d mp3c_execute_printf(%s,%d)", __FILE__,__LINE__,fmt,want_meta);
  if (!c) { error("no connection"); return NULL; }
  va_start(ap, fmt);
@@ -210,11 +211,12 @@ char*** mp3c_execute_printf(PGconn* c,int* row, int* col, char* errstr, int want
  return ret;
 }
 
-char*** mem_p3c_execute_printf(struct mem_list** mem, PGconn* c,int* row, int* col, char* errstr, int want_meta, char* fmt,...)
+
+CHAR*** mem_p3c_execute_printf(struct mem_list** mem, PGconn* c,INT* row, INT* col, CHAR* errstr, INT want_meta, CHAR* fmt,...)
 {
  va_list ap;
- char*** ret;
- char msg[MAX_QUERY_LENGTH*2];
+ CHAR*** ret;
+ CHAR msg[MAX_QUERY_LENGTH*2];
  debug("%s:%d mem_p3c_execute_printf(%s,%d)", __FILE__,__LINE__,fmt,want_meta);
  if (!c) { error("no connection"); return NULL; }
  va_start(ap, fmt);
@@ -224,11 +226,12 @@ char*** mem_p3c_execute_printf(struct mem_list** mem, PGconn* c,int* row, int* c
  return ret;
 }
 
-void dmem_execute_printf(struct dmem_list** mem, PGconn* c, char**** res, int* row, int* col, char* errstr, int want_meta, char* file, int line, char* fmt,...)
+
+void dmem_execute_printf(struct dmem_list** mem, PGconn* c, CHAR**** res, INT* row, INT* col, CHAR* errstr, INT want_meta, CHAR* file, INT line, CHAR* fmt,...)
 {
- int stat;
+ INT stat;
  va_list ap;
- char query[MAX_QUERY_LENGTH*2];
+ CHAR query[MAX_QUERY_LENGTH*2];
  PGresult* r;
  debug("%s:%d mem_execute_printf(%s,%d) called from %s:%d", __FILE__,__LINE__,fmt,want_meta,file,line);
  va_start(ap, fmt);
@@ -271,11 +274,12 @@ void dmem_execute_printf(struct dmem_list** mem, PGconn* c, char**** res, int* r
  clear_result(&r);
 }
 
-void mem_execute_printf(struct mem_list** mem, PGconn* c, char**** res, int* row, int* col, char* errstr, int want_meta, char* fmt,...)
+
+void mem_execute_printf(struct mem_list** mem, PGconn* c, CHAR**** res, INT* row, INT* col, CHAR* errstr, INT want_meta, CHAR* fmt,...)
 {
- int stat;
+ INT stat;
  va_list ap;
- char query[MAX_QUERY_LENGTH*2];
+ CHAR query[MAX_QUERY_LENGTH*2];
  PGresult* r;
  debug("%s:%d mem_execute_printf(%s,%d)", __FILE__,__LINE__,fmt,want_meta);
  va_start(ap, fmt);
@@ -318,11 +322,12 @@ void mem_execute_printf(struct mem_list** mem, PGconn* c, char**** res, int* row
  clear_result(&r);
 }
 
-void dmexecute_printf(PGconn* c, char**** res, int* row, int* col, char* errstr, int want_meta, char* file, int line, char* fmt,...)
+
+void dmexecute_printf(PGconn* c, CHAR**** res, INT* row, INT* col, CHAR* errstr, INT want_meta, CHAR* file, INT line, CHAR* fmt,...)
 {
- int stat;
+ INT stat;
  va_list ap;
- char query[MAX_QUERY_LENGTH*2];
+ CHAR query[MAX_QUERY_LENGTH*2];
  PGresult* r;
  debug("%s:%d dmexecute_printf(%s,%d) called from %s:%d", __FILE__,__LINE__,fmt,want_meta,file,line);
  va_start(ap, fmt);
@@ -365,11 +370,12 @@ void dmexecute_printf(PGconn* c, char**** res, int* row, int* col, char* errstr,
  clear_result(&r);
 }
 
-void mexecute_printf(PGconn* c, char**** res, int* row, int* col, char* errstr, int want_meta, char* fmt,...)
+
+void mexecute_printf(PGconn* c, CHAR**** res, INT* row, INT* col, CHAR* errstr, INT want_meta, CHAR* fmt,...)
 {
- int stat;
+ INT stat;
  va_list ap;
- char query[MAX_QUERY_LENGTH*2];
+ CHAR query[MAX_QUERY_LENGTH*2];
  PGresult* r;
  debug("%s:%d mexecute_printf(%s,%d)", __FILE__,__LINE__,fmt,want_meta);
  va_start(ap, fmt);
@@ -412,11 +418,12 @@ void mexecute_printf(PGconn* c, char**** res, int* row, int* col, char* errstr,
  clear_result(&r);
 }
 
-void execute_printf(PGconn* c, char**** res, int* row, int* col, char* errstr, int want_meta, char* fmt,...)
+
+void execute_printf(PGconn* c, CHAR**** res, INT* row, INT* col, CHAR* errstr, INT want_meta, CHAR* fmt,...)
 {
- int stat;
+ INT stat;
  va_list ap;
- char query[MAX_QUERY_LENGTH*2];
+ CHAR query[MAX_QUERY_LENGTH*2];
  PGresult* r;
  debug("%s:%d execute_printf(%s,%d)", __FILE__,__LINE__,fmt,want_meta);
  va_start(ap, fmt);
@@ -460,11 +467,11 @@ void execute_printf(PGconn* c, char**** res, int* row, int* col, char* errstr, i
 }
 
 
-void execute_printf_query(PGconn* c,PGresult** r, char* errstr, char* fmt,...)
+void execute_printf_query(PGconn* c,PGresult** r, CHAR* errstr, CHAR* fmt,...)
 {
- int stat;
+ INT stat;
  va_list ap;
- char query[MAX_QUERY_LENGTH*2];
+ CHAR query[MAX_QUERY_LENGTH*2];
  debug("%s:%d execute_printf_query(%s)", __FILE__,__LINE__,fmt);
  va_start(ap, fmt);
  vsprintf(query,fmt,ap);
@@ -498,12 +505,13 @@ void execute_printf_query(PGconn* c,PGresult** r, char* errstr, char* fmt,...)
    }
 }
 
-int get_db_tables(PGconn* c, char*** dat, int* ntabs, int want_all_elems)
+
+INT get_db_tables(PGconn* c, CHAR*** dat, INT* ntabs, INT want_all_elems)
 {
- char*** res;
- char** data;
- int row,col,i;
- char errstr[MAX_QUERY_LENGTH+100];
+ CHAR*** res;
+ CHAR** data;
+ INT row,col,i;
+ CHAR errstr[MAX_QUERY_LENGTH+100];
  data = *dat;
  debug("%s:%d get_db_tables(%d)", __FILE__,__LINE__,want_all_elems);
  if (!want_all_elems)
@@ -515,23 +523,24 @@ int get_db_tables(PGconn* c, char*** dat, int* ntabs, int want_all_elems)
  if (!res) { *dat=NULL; ntabs=0; warn("empty result get_db_tables: %s:%d", __FILE__,__LINE__); return 0; }
  if (col<0 || row<0) { printf("errstr=%s\n", errstr); (*dat) = NULL; *ntabs=-1; return 0; }
  *ntabs = row;
- data = (char**)malloc(row*sizeof(char*));
- for (i=0;i<row;i++) 
-   { 
-    data[i] = malloc(strlen(res[i][0])+1); 
-    strcpy(data[i], res[i][0]); 
+ data = (CHAR**)malloc(row*sizeof(CHAR*));
+ for (i=0;i<row;i++)
+   {
+    data[i] = malloc(strlen(res[i][0])+1);
+    strcpy(data[i], res[i][0]);
    }
  free_p3c(&res, row, col);		/* FIXME CHECK IT */
  *dat = data;
  return 1;
 }
 
-int dmget_db_tables(PGconn* c, char*** dat, int* ntabs, int want_all_elems, char* file, int line)
+
+INT dmget_db_tables(PGconn* c, CHAR*** dat, INT* ntabs, INT want_all_elems, CHAR* file, INT line)
 {
- char*** res;
- char** data;
- int row,col,i;
- char errstr[MAX_QUERY_LENGTH+100];
+ CHAR*** res;
+ CHAR** data;
+ INT row,col,i;
+ CHAR errstr[MAX_QUERY_LENGTH+100];
  data = *dat;
  debug("%s:%d mget_db_tables(%d) called from %s:%d", __FILE__,__LINE__,want_all_elems,file,line);
  if (!want_all_elems)
@@ -543,23 +552,24 @@ int dmget_db_tables(PGconn* c, char*** dat, int* ntabs, int want_all_elems, char
  if (!res) { *dat=NULL; ntabs=0; warn("empty result get_db_tables: %s:%d", __FILE__,__LINE__); return 0; }
  if (col<0 || row<0) { printf("errstr=%s\n", errstr); (*dat) = NULL; *ntabs=-1; return 0; }
  *ntabs = row;
- data = (char**)dmmalloc(row*sizeof(char*), HERE);
- for (i=0;i<row;i++) 
-   { 
-    data[i] = dmmalloc(strlen(res[i][0])+1, HERE); 
-    strcpy(data[i], res[i][0]); 
+ data = (CHAR**)dmmalloc(row*sizeof(CHAR*), HERE);
+ for (i=0;i<row;i++)
+   {
+    data[i] = dmmalloc(strlen(res[i][0])+1, HERE);
+    strcpy(data[i], res[i][0]);
    }
  dmfree_p3c(&res, row, col, HERE);		/* FIXME CHECK IT */
  *dat = data;
  return 1;
 }
 
-int mget_db_tables(PGconn* c, char*** dat, int* ntabs, int want_all_elems)
+
+INT mget_db_tables(PGconn* c, CHAR*** dat, INT* ntabs, INT want_all_elems)
 {
- char*** res;
- char** data;
- int row,col,i;
- char errstr[MAX_QUERY_LENGTH+100];
+ CHAR*** res;
+ CHAR** data;
+ INT row,col,i;
+ CHAR errstr[MAX_QUERY_LENGTH+100];
  data = *dat;
  debug("%s:%d mget_db_tables(%d)", __FILE__,__LINE__,want_all_elems);
  if (!want_all_elems)
@@ -571,23 +581,24 @@ int mget_db_tables(PGconn* c, char*** dat, int* ntabs, int want_all_elems)
  if (!res) { *dat=NULL; ntabs=0; warn("empty result get_db_tables: %s:%d", __FILE__,__LINE__); return 0; }
  if (col<0 || row<0) { printf("errstr=%s\n", errstr); (*dat) = NULL; *ntabs=-1; return 0; }
  *ntabs = row;
- data = (char**)mmalloc(row*sizeof(char*));
- for (i=0;i<row;i++) 
-   { 
-    data[i] = mmalloc(strlen(res[i][0])+1); 
-    strcpy(data[i], res[i][0]); 
+ data = (CHAR**)mmalloc(row*sizeof(CHAR*));
+ for (i=0;i<row;i++)
+   {
+    data[i] = mmalloc(strlen(res[i][0])+1);
+    strcpy(data[i], res[i][0]);
    }
  mfree_p3c(&res, row, col);		/* FIXME CHECK IT */
  *dat = data;
  return 1;
 }
 
-int mem_get_db_tables(struct mem_list** mem, PGconn* c, char*** dat, int* ntabs, int want_all_elems)
+
+INT mem_get_db_tables(struct mem_list** mem, PGconn* c, CHAR*** dat, INT* ntabs, INT want_all_elems)
 {
- char*** res;
- char** data;
- int row,col,i;
- char errstr[MAX_QUERY_LENGTH+100];
+ CHAR*** res;
+ CHAR** data;
+ INT row,col,i;
+ CHAR errstr[MAX_QUERY_LENGTH+100];
  data = *dat;
  debug("%s:%d mem_get_db_tables(%d)", __FILE__,__LINE__,want_all_elems);
  if (!want_all_elems)
@@ -599,23 +610,24 @@ int mem_get_db_tables(struct mem_list** mem, PGconn* c, char*** dat, int* ntabs,
  if (!res) { *dat=NULL; ntabs=0; warn("empty result get_db_tables: %s:%d", __FILE__,__LINE__); return 0; }
  if (col<0 || row<0) { debug("errstr=%s\n", errstr); (*dat) = NULL; *ntabs=-1; return 0; }
  *ntabs = row;
- data = (char**)smalloc(mem, row*sizeof(char*));
- for (i=0;i<row;i++) 
-   { 
-    data[i] = smalloc(mem, strlen(res[i][0])+1); 
-    strcpy(data[i], res[i][0]); 
+ data = (CHAR**)smalloc(mem, row*sizeof(CHAR*));
+ for (i=0;i<row;i++)
+   {
+    data[i] = smalloc(mem, strlen(res[i][0])+1);
+    strcpy(data[i], res[i][0]);
    }
  mem_free_p3c(mem, &res, row, col);
  *dat = data;
  return 1;
 }
 
-int dmem_get_db_tables(struct dmem_list** mem, PGconn* c, char*** dat, int* ntabs, int want_all_elems,char* file, int line)
+
+INT dmem_get_db_tables(struct dmem_list** mem, PGconn* c, CHAR*** dat, INT* ntabs, INT want_all_elems,CHAR* file, INT line)
 {
- char*** res;
- char** data;
- int row,col,i;
- char errstr[MAX_QUERY_LENGTH+100];
+ CHAR*** res;
+ CHAR** data;
+ INT row,col,i;
+ CHAR errstr[MAX_QUERY_LENGTH+100];
  data = *dat;
  debug("%s:%d mem_get_db_tables(%d) called from %s:%d", __FILE__,__LINE__,want_all_elems,file,line);
  if (!want_all_elems)
@@ -627,11 +639,11 @@ int dmem_get_db_tables(struct dmem_list** mem, PGconn* c, char*** dat, int* ntab
  if (!res) { *dat=NULL; ntabs=0; warn("empty result get_db_tables: %s:%d", __FILE__,__LINE__); return 0; }
  if (col<0 || row<0) { debug("errstr=%s\n", errstr); (*dat) = NULL; *ntabs=-1; return 0; }
  *ntabs = row;
- data = (char**)dsmalloc(mem, row*sizeof(char*), HERE);
- for (i=0;i<row;i++) 
-   { 
-    data[i] = dsmalloc(mem, strlen(res[i][0])+1, HERE); 
-    strcpy(data[i], res[i][0]); 
+ data = (CHAR**)dsmalloc(mem, row*sizeof(CHAR*), HERE);
+ for (i=0;i<row;i++)
+   {
+    data[i] = dsmalloc(mem, strlen(res[i][0])+1, HERE);
+    strcpy(data[i], res[i][0]);
    }
  dmem_free_p3c(mem, &res, row, col, HERE);
  *dat = data;
@@ -639,10 +651,10 @@ int dmem_get_db_tables(struct dmem_list** mem, PGconn* c, char*** dat, int* ntab
 }
 
 
-PGresult* execute_query(PGconn* c,  char* query)
+PGresult* execute_query(PGconn* c,  CHAR* query)
 {
  PGresult* r;
- int stat;
+ INT stat;
  debug("%s:%d execute_query(%s)", __FILE__,__LINE__,query);
  debug("SQLquery: \"%s\"\n", query);
  if (!query)
@@ -668,31 +680,33 @@ PGresult* execute_query(PGconn* c,  char* query)
  return r;
 }
 
-char*** dmem_pgres2pc3(struct dmem_list** mem, const PGresult* r, int* a, int* b, int want_meta, char* file, int line)
+
+CHAR*** dmem_pgres2pc3(struct dmem_list** mem, const PGresult* r, INT* a, INT* b, INT want_meta, CHAR* file, INT line)
 {
- char*** result;
- char* tmp;
- int i,j;
+ CHAR*** result;
+ CHAR* tmp;
+ INT i,j;
  debug("%s:%d dmem_pgres2pc3() called from %s:%d", __FILE__,__LINE__,file,line);
  debug("mem_pgres2pc3: %p\n", (void*)r);
  if (!r) return 0;
  i=j=0;
  *a = PQntuples(r);
  *b = PQnfields(r);
- if (!(*a) || !(*b)) 
-   { 
-    warn("no cols or no rows (%s:%d)", __FILE__,__LINE__); 
-    return NULL; 
+ if ((!(*a) || !(*b)) && !want_meta)
+ //if (!(*a) || !(*b))
+   {
+    warn("no cols or no rows (%s:%d)", __FILE__,__LINE__);
+    return NULL;
    }
  /*printf("ntuples=%d,%d\n", *a,*b);*/
  /*if (!(*a) || !(*b)) { warn("no cols or no rows (%s:%d)", __FILE__,__LINE__); return NULL; }*/
  if (!want_meta)
  {
- result = (char***)dsmalloc(mem, (*a)*sizeof(char**), HERE);
+ result = (CHAR***)dsmalloc(mem, (*a)*sizeof(CHAR**), HERE);
  if (!result) { ERR error("smalloc error\n"); return NULL; }
  for (i=0;i<*a;i++)
    {
-    result[i] = (char**)dsmalloc(mem, (*b)*sizeof(char*), HERE);
+    result[i] = (CHAR**)dsmalloc(mem, (*b)*sizeof(CHAR*), HERE);
     if (!result[i])
       {
        for (j=0;j<i;j++) dsfree(mem, (void**)(&result[i]), HERE);
@@ -703,17 +717,17 @@ char*** dmem_pgres2pc3(struct dmem_list** mem, const PGresult* r, int* a, int* b
  for (i=0;i<*a;i++)
  for (j=0;j<*b;j++)
    {
-    result[i][j] = (char*)dsmalloc(mem, PQgetlength(r,i,j)+1, HERE);
+    result[i][j] = (CHAR*)dsmalloc(mem, PQgetlength(r,i,j)+1, HERE);
     if (!result[i][j])
       {
        dmem_free_p3c(mem, &result,*a,*b, HERE);
        ERR error("malloc_ij error\n");
        return NULL;
       }
-    strcpy(result[i][j], (char*)PQgetvalue(r,i,j));
+    strcpy(result[i][j], (CHAR*)PQgetvalue(r,i,j));
    }
 /*#ifdef DEBUG
-  debug("char*** result is (%d:%d):\n",i,j);
+  debug("CHAR*** result is (%d:%d):\n",i,j);
   for (i=0;i<*a;i++)
     {
      for (j=0;j<*b;j++) debug("%-15s ", result[i][j]);
@@ -726,11 +740,11 @@ char*** dmem_pgres2pc3(struct dmem_list** mem, const PGresult* r, int* a, int* b
  {
   if ((*b)<=0) return NULL;		/*FIXME is it OK? */
  (*a)++;
- result = (char***)dsmalloc(mem, (*a)*sizeof(char**), HERE);
+ result = (CHAR***)dsmalloc(mem, (*a)*sizeof(CHAR**), HERE);
  if (!result) { ERR error("smalloc error\n"); return NULL; }
  for (i=0;i<*a;i++)
    {
-    result[i] = (char**)dsmalloc(mem, (*b)*sizeof(char*), HERE);
+    result[i] = (CHAR**)dsmalloc(mem, (*b)*sizeof(CHAR*), HERE);
     if (!result[i])
       {
        for (j=0;j<i;j++) dsfree(mem, (void**)(&result[i]), HERE);
@@ -741,51 +755,53 @@ char*** dmem_pgres2pc3(struct dmem_list** mem, const PGresult* r, int* a, int* b
  for (i=0;i<*b;i++)
    {
     tmp = PQfname(r,i);
-    result[0][i] = (char*)dsmalloc(mem, strlen(tmp)+1, HERE);
+    result[0][i] = (CHAR*)dsmalloc(mem, strlen(tmp)+1, HERE);
     if (!result[0][i]) { ERR error("smalloc_namej error"); return NULL; }
     strcpy(result[0][i], tmp);
    }
  for (i=1;i<*a;i++)
  for (j=0;j<*b;j++)
    {
-    result[i][j] = (char*)dsmalloc(mem, PQgetlength(r,i-1,j)+1, HERE);
+    result[i][j] = (CHAR*)dsmalloc(mem, PQgetlength(r,i-1,j)+1, HERE);
     if (!result[i][j])
       {
        dmem_free_p3c(mem, &result,*a,*b, HERE);
        ERR error("malloc_ij error\n");
        return NULL;
       }
-    strcpy(result[i][j], (char*)PQgetvalue(r,i-1,j));
+    strcpy(result[i][j], (CHAR*)PQgetvalue(r,i-1,j));
    }
   return result;
  }
 }
 
-char*** mem_pgres2pc3(struct mem_list** mem, const PGresult* r, int* a, int* b, int want_meta)
+
+CHAR*** mem_pgres2pc3(struct mem_list** mem, const PGresult* r, INT* a, INT* b, INT want_meta)
 {
- char*** result;
- char* tmp;
- int i,j;
+ CHAR*** result;
+ CHAR* tmp;
+ INT i,j;
  debug("%s:%d mem_pgres2pc3()", __FILE__,__LINE__);
  debug("mem_pgres2pc3: %p\n", (void*)r);
  if (!r) return 0;
  i=j=0;
  *a = PQntuples(r);
  *b = PQnfields(r);
- if (!(*a) || !(*b)) 
-   { 
-    warn("no cols or no rows (%s:%d)", __FILE__,__LINE__); 
-    return NULL; 
+ if ((!(*a) || !(*b)) && !want_meta)
+ //if (!(*a) || !(*b))
+   {
+    warn("no cols or no rows (%s:%d)", __FILE__,__LINE__);
+    return NULL;
    }
  /*printf("ntuples=%d,%d\n", *a,*b);*/
  /*if (!(*a) || !(*b)) { warn("no cols or no rows (%s:%d)", __FILE__,__LINE__); return NULL; }*/
  if (!want_meta)
  {
- result = (char***)smalloc(mem, (*a)*sizeof(char**));
+ result = (CHAR***)smalloc(mem, (*a)*sizeof(CHAR**));
  if (!result) { ERR error("smalloc error\n"); return NULL; }
  for (i=0;i<*a;i++)
    {
-    result[i] = (char**)smalloc(mem, (*b)*sizeof(char*));
+    result[i] = (CHAR**)smalloc(mem, (*b)*sizeof(CHAR*));
     if (!result[i])
       {
        for (j=0;j<i;j++) sfree(mem, (void**)(&result[i]));
@@ -796,17 +812,17 @@ char*** mem_pgres2pc3(struct mem_list** mem, const PGresult* r, int* a, int* b,
  for (i=0;i<*a;i++)
  for (j=0;j<*b;j++)
    {
-    result[i][j] = (char*)smalloc(mem, PQgetlength(r,i,j)+1);
+    result[i][j] = (CHAR*)smalloc(mem, PQgetlength(r,i,j)+1);
     if (!result[i][j])
       {
        mem_free_p3c(mem, &result,*a,*b);
        ERR error("malloc_ij error\n");
        return NULL;
       }
-    strcpy(result[i][j], (char*)PQgetvalue(r,i,j));
+    strcpy(result[i][j], (CHAR*)PQgetvalue(r,i,j));
    }
 /*#ifdef DEBUG
-  debug("char*** result is (%d:%d):\n",i,j);
+  debug("CHAR*** result is (%d:%d):\n",i,j);
   for (i=0;i<*a;i++)
     {
      for (j=0;j<*b;j++) debug("%-15s ", result[i][j]);
@@ -819,11 +835,11 @@ char*** mem_pgres2pc3(struct mem_list** mem, const PGresult* r, int* a, int* b,
  {
   if ((*b)<=0) return NULL;		/*FIXME is it OK? */
  (*a)++;
- result = (char***)smalloc(mem, (*a)*sizeof(char**));
+ result = (CHAR***)smalloc(mem, (*a)*sizeof(CHAR**));
  if (!result) { ERR error("smalloc error\n"); return NULL; }
  for (i=0;i<*a;i++)
    {
-    result[i] = (char**)smalloc(mem, (*b)*sizeof(char*));
+    result[i] = (CHAR**)smalloc(mem, (*b)*sizeof(CHAR*));
     if (!result[i])
       {
        for (j=0;j<i;j++) sfree(mem, (void**)(&result[i]));
@@ -834,31 +850,32 @@ char*** mem_pgres2pc3(struct mem_list** mem, const PGresult* r, int* a, int* b,
  for (i=0;i<*b;i++)
    {
     tmp = PQfname(r,i);
-    result[0][i] = (char*)smalloc(mem, strlen(tmp)+1);
+    result[0][i] = (CHAR*)smalloc(mem, strlen(tmp)+1);
     if (!result[0][i]) { ERR error("smalloc_namej error"); return NULL; }
     strcpy(result[0][i], tmp);
    }
  for (i=1;i<*a;i++)
  for (j=0;j<*b;j++)
    {
-    result[i][j] = (char*)smalloc(mem, PQgetlength(r,i-1,j)+1);
+    result[i][j] = (CHAR*)smalloc(mem, PQgetlength(r,i-1,j)+1);
     if (!result[i][j])
       {
        mem_free_p3c(mem, &result,*a,*b);
        ERR error("malloc_ij error\n");
        return NULL;
       }
-    strcpy(result[i][j], (char*)PQgetvalue(r,i-1,j));
+    strcpy(result[i][j], (CHAR*)PQgetvalue(r,i-1,j));
    }
   return result;
  }
 }
 
-char*** dmpgres2pc3(const PGresult* r, int* a, int* b, int want_meta, char* file, int line)
+
+CHAR*** dmpgres2pc3(const PGresult* r, INT* a, INT* b, INT want_meta, CHAR* file, INT line)
 {
- char*** result;
- char* tmp;
- int i,j;
+ CHAR*** result;
+ CHAR* tmp;
+ INT i,j;
  debug("%s:%d dmpgres2pc3() called from %s:%d", __FILE__,__LINE__,file,line);
  debug("dmpgres2pc3: %p\n", (void*)r);
  if (!r) return 0;
@@ -866,22 +883,23 @@ char*** dmpgres2pc3(const PGresult* r, int* a, int* b, int want_meta, char* file
  *a = PQntuples(r);
  *b = PQnfields(r);
  /*if (!(*a) || !(*b)) { warn("no cols or no rows (%s:%d)", __FILE__,__LINE__); return NULL; }*/
- if (!(*a) || !(*b)) 
-   { 
-    warn("no cols or no rows (%s:%d)", __FILE__,__LINE__); 
-    return NULL; 
+ if ((!(*a) || !(*b)) && !want_meta)
+ //if (!(*a) || !(*b))
+   {
+    warn("no cols or no rows (%s:%d)", __FILE__,__LINE__);
+    return NULL;
    }
  /*printf("ntuples=%d,%d\n", *a,*b);*/
  if (!want_meta)
  {
- result = (char***)dmmalloc((*a)*sizeof(char**), HERE);
+ result = (CHAR***)dmmalloc((*a)*sizeof(CHAR**), HERE);
  if (!result) { ERR error("mmalloc error\n"); return NULL; }
  for (i=0;i<*a;i++)
   {
-    result[i] = (char**)dmmalloc((*b)*sizeof(char*), HERE);
+    result[i] = (CHAR**)dmmalloc((*b)*sizeof(CHAR*), HERE);
     if (!result[i])
       {
-       for (j=0;j<i;j++) 
+       for (j=0;j<i;j++)
        {
 	 dmfree((void**)(&result[i]), HERE);
        }
@@ -892,17 +910,17 @@ char*** dmpgres2pc3(const PGresult* r, int* a, int* b, int want_meta, char* file
  for (i=0;i<*a;i++)
  for (j=0;j<*b;j++)
    {
-    result[i][j] = (char*)dmmalloc(PQgetlength(r,i,j)+1, HERE);
+    result[i][j] = (CHAR*)dmmalloc(PQgetlength(r,i,j)+1, HERE);
     if (!result[i][j])
       {
        dmfree_p3c(&result,*a,*b, HERE);
        ERR error("mmalloc_ij error\n");
        return NULL;
       }
-    strcpy(result[i][j], (char*)PQgetvalue(r,i,j));
+    strcpy(result[i][j], (CHAR*)PQgetvalue(r,i,j));
    }
 /*#ifdef DEBUG
-  debug("char*** result is (%d:%d):\n",i,j);
+  debug("CHAR*** result is (%d:%d):\n",i,j);
   for (i=0;i<*a;i++)
     {
      for (j=0;j<*b;j++) debug("%-15s ", result[i][j]);
@@ -913,16 +931,16 @@ char*** dmpgres2pc3(const PGresult* r, int* a, int* b, int want_meta, char* file
  }
  else
  {
-  if ((*b)<=0) return NULL;		/*FIXME is it OK? */
+  if ((*b)<0) return NULL;		/*FIXME is it OK? */
  (*a)++;
- result = (char***)dmmalloc((*a)*sizeof(char**), HERE);
+ result = (CHAR***)dmmalloc((*a)*sizeof(CHAR**), HERE);
  if (!result) { ERR error("mmalloc error\n"); return NULL; }
  for (i=0;i<*a;i++)
    {
-    result[i] = (char**)dmmalloc((*b)*sizeof(char*), HERE);
+    result[i] = (CHAR**)dmmalloc((*b)*sizeof(CHAR*), HERE);
     if (!result[i])
       {
-       for (j=0;j<i;j++) 
+       for (j=0;j<i;j++)
          {
 	  dmfree((void**)(&result[i]), HERE);
 	 }
@@ -933,31 +951,32 @@ char*** dmpgres2pc3(const PGresult* r, int* a, int* b, int want_meta, char* file
  for (i=0;i<*b;i++)
    {
     tmp = PQfname(r,i);
-    result[0][i] = (char*)dmmalloc(strlen(tmp)+1, HERE);
+    result[0][i] = (CHAR*)dmmalloc(strlen(tmp)+1, HERE);
     if (!result[0][i]) { ERR error("mmalloc_namej error"); return NULL; }
     strcpy(result[0][i], tmp);
    }
  for (i=1;i<*a;i++)
  for (j=0;j<*b;j++)
    {
-    result[i][j] = (char*)dmmalloc(PQgetlength(r,i-1,j)+1, HERE);
+    result[i][j] = (CHAR*)dmmalloc(PQgetlength(r,i-1,j)+1, HERE);
     if (!result[i][j])
       {
        dmfree_p3c(&result,*a,*b, HERE);
        ERR error("mmalloc_ij error\n");
        return NULL;
       }
-    strcpy(result[i][j], (char*)PQgetvalue(r,i-1,j));
+    strcpy(result[i][j], (CHAR*)PQgetvalue(r,i-1,j));
    }
   return result;
  }
 }
 
-char*** mpgres2pc3(const PGresult* r, int* a, int* b, int want_meta)
+
+CHAR*** mpgres2pc3(const PGresult* r, INT* a, INT* b, INT want_meta)
 {
- char*** result;
- char* tmp;
- int i,j;
+ CHAR*** result;
+ CHAR* tmp;
+ INT i,j;
  debug("%s:%d mpgres2pc3()", __FILE__,__LINE__);
  debug("mpgres2pc3: %p\n", (void*)r);
  if (!r) return 0;
@@ -965,22 +984,23 @@ char*** mpgres2pc3(const PGresult* r, int* a, int* b, int want_meta)
  *a = PQntuples(r);
  *b = PQnfields(r);
  /*if (!(*a) || !(*b)) { warn("no cols or no rows (%s:%d)", __FILE__,__LINE__); return NULL; }*/
- if (!(*a) || !(*b)) 
-   { 
-    warn("no cols or no rows (%s:%d)", __FILE__,__LINE__); 
-    return NULL; 
+ if ((!(*a) || !(*b)) && !want_meta)
+ //if (!(*a) || !(*b))
+   {
+    warn("no cols or no rows (%s:%d)", __FILE__,__LINE__);
+    return NULL;
    }
  /*printf("ntuples=%d,%d\n", *a,*b);*/
  if (!want_meta)
  {
- result = (char***)mmalloc((*a)*sizeof(char**));
+ result = (CHAR***)mmalloc((*a)*sizeof(CHAR**));
  if (!result) { ERR error("mmalloc error\n"); return NULL; }
  for (i=0;i<*a;i++)
   {
-    result[i] = (char**)mmalloc((*b)*sizeof(char*));
+    result[i] = (CHAR**)mmalloc((*b)*sizeof(CHAR*));
     if (!result[i])
       {
-       for (j=0;j<i;j++) 
+       for (j=0;j<i;j++)
        {
 	 mfree((void**)(&result[i]));
        }
@@ -991,17 +1011,17 @@ char*** mpgres2pc3(const PGresult* r, int* a, int* b, int want_meta)
  for (i=0;i<*a;i++)
  for (j=0;j<*b;j++)
    {
-    result[i][j] = (char*)mmalloc(PQgetlength(r,i,j)+1);
+    result[i][j] = (CHAR*)mmalloc(PQgetlength(r,i,j)+1);
     if (!result[i][j])
       {
        mfree_p3c(&result,*a,*b);
        ERR error("mmalloc_ij error\n");
        return NULL;
       }
-    strcpy(result[i][j], (char*)PQgetvalue(r,i,j));
+    strcpy(result[i][j], (CHAR*)PQgetvalue(r,i,j));
    }
 /*#ifdef DEBUG
-  debug("char*** result is (%d:%d):\n",i,j);
+  debug("CHAR*** result is (%d:%d):\n",i,j);
   for (i=0;i<*a;i++)
     {
      for (j=0;j<*b;j++) debug("%-15s ", result[i][j]);
@@ -1014,14 +1034,14 @@ char*** mpgres2pc3(const PGresult* r, int* a, int* b, int want_meta)
  {
   if ((*b)<=0) return NULL;		/*FIXME is it OK? */
  (*a)++;
- result = (char***)mmalloc((*a)*sizeof(char**));
+ result = (CHAR***)mmalloc((*a)*sizeof(CHAR**));
  if (!result) { ERR error("mmalloc error\n"); return NULL; }
  for (i=0;i<*a;i++)
    {
-    result[i] = (char**)mmalloc((*b)*sizeof(char*));
+    result[i] = (CHAR**)mmalloc((*b)*sizeof(CHAR*));
     if (!result[i])
       {
-       for (j=0;j<i;j++) 
+       for (j=0;j<i;j++)
          {
 	  mfree((void**)(&result[i]));
 	 }
@@ -1032,50 +1052,52 @@ char*** mpgres2pc3(const PGresult* r, int* a, int* b, int want_meta)
  for (i=0;i<*b;i++)
    {
     tmp = PQfname(r,i);
-    result[0][i] = (char*)mmalloc(strlen(tmp)+1);
+    result[0][i] = (CHAR*)mmalloc(strlen(tmp)+1);
     if (!result[0][i]) { ERR error("mmalloc_namej error"); return NULL; }
     strcpy(result[0][i], tmp);
    }
  for (i=1;i<*a;i++)
  for (j=0;j<*b;j++)
    {
-    result[i][j] = (char*)mmalloc(PQgetlength(r,i-1,j)+1);
+    result[i][j] = (CHAR*)mmalloc(PQgetlength(r,i-1,j)+1);
     if (!result[i][j])
       {
        mfree_p3c(&result,*a,*b);
        ERR error("mmalloc_ij error\n");
        return NULL;
       }
-    strcpy(result[i][j], (char*)PQgetvalue(r,i-1,j));
+    strcpy(result[i][j], (CHAR*)PQgetvalue(r,i-1,j));
    }
   return result;
  }
 }
 
-char*** pgres2pc3(const PGresult* r, int* a, int* b, int want_meta)
+
+CHAR*** pgres2pc3(const PGresult* r, INT* a, INT* b, INT want_meta)
 {
- char*** result;
- char* tmp;
- int i,j;
+ CHAR*** result;
+ CHAR* tmp;
+ INT i,j;
  debug("%s:%d pgres2pc3()", __FILE__,__LINE__);
  debug("pgres2pc3: %p\n", (void*)r);
  if (!r) return 0;
  i=j=0;
  *a = PQntuples(r);
  *b = PQnfields(r);
- if (!(*a) || !(*b)) 
-   { 
-    warn("no cols or no rows (%s:%d)", __FILE__,__LINE__); 
-    return NULL; 
+ if ((!(*a) || !(*b)) && !want_meta)
+ //if (!(*a) || !(*b))
+   {
+    warn("no cols or no rows (%s:%d)", __FILE__,__LINE__);
+    return NULL;
    }
  /*printf("ntuples=%d,%d\n", *a,*b);*/
  if (!want_meta)
  {
- result = (char***)malloc((*a)*sizeof(char**));
+ result = (CHAR***)malloc((*a)*sizeof(CHAR**));
  if (!result) { ERR error("malloc error\n"); return NULL; }
  for (i=0;i<*a;i++)
    {
-    result[i] = (char**)malloc((*b)*sizeof(char*));
+    result[i] = (CHAR**)malloc((*b)*sizeof(CHAR*));
     if (!result[i])
       {
        for (j=0;j<i;j++) free(result[i]);
@@ -1086,17 +1108,17 @@ char*** pgres2pc3(const PGresult* r, int* a, int* b, int want_meta)
  for (i=0;i<*a;i++)
  for (j=0;j<*b;j++)
    {
-    result[i][j] = (char*)malloc(PQgetlength(r,i,j)+1);
+    result[i][j] = (CHAR*)malloc(PQgetlength(r,i,j)+1);
     if (!result[i][j])
       {
        free_p3c(&result,*a,*b);
        ERR error("malloc_ij error\n");
        return NULL;
       }
-    strcpy(result[i][j], (char*)PQgetvalue(r,i,j));
+    strcpy(result[i][j], (CHAR*)PQgetvalue(r,i,j));
    }
 /*#ifdef DEBUG
-  debug("char*** result is (%d:%d):\n",i,j);
+  debug("CHAR*** result is (%d:%d):\n",i,j);
   for (i=0;i<*a;i++)
     {
      for (j=0;j<*b;j++) debug("%-15s ", result[i][j]);
@@ -1109,11 +1131,11 @@ char*** pgres2pc3(const PGresult* r, int* a, int* b, int want_meta)
  {
   if ((*b)<=0) return NULL;		/*FIXME is it OK? */
  (*a)++;
- result = (char***)malloc((*a)*sizeof(char**));
+ result = (CHAR***)malloc((*a)*sizeof(CHAR**));
  if (!result) { ERR error("malloc error\n"); return NULL; }
  for (i=0;i<*a;i++)
    {
-    result[i] = (char**)malloc((*b)*sizeof(char*));
+    result[i] = (CHAR**)malloc((*b)*sizeof(CHAR*));
     if (!result[i])
       {
        for (j=0;j<i;j++) free(result[i]);
@@ -1124,29 +1146,30 @@ char*** pgres2pc3(const PGresult* r, int* a, int* b, int want_meta)
  for (i=0;i<*b;i++)
    {
     tmp = PQfname(r,i);
-    result[0][i] = (char*)malloc(strlen(tmp)+1);
+    result[0][i] = (CHAR*)malloc(strlen(tmp)+1);
     if (!result[0][i]) { ERR error("malloc_namej error"); return NULL; }
     strcpy(result[0][i], tmp);
    }
  for (i=1;i<*a;i++)
  for (j=0;j<*b;j++)
    {
-    result[i][j] = (char*)malloc(PQgetlength(r,i-1,j)+1);
+    result[i][j] = (CHAR*)malloc(PQgetlength(r,i-1,j)+1);
     if (!result[i][j])
       {
        free_p3c(&result,*a,*b);
        ERR error("malloc_ij error\n");
        return NULL;
       }
-    strcpy(result[i][j], (char*)PQgetvalue(r,i-1,j));
+    strcpy(result[i][j], (CHAR*)PQgetvalue(r,i-1,j));
    }
   return result;
  }
 }
 
-void mem_free_p3c(struct mem_list** mem, char**** p, int a, int b)
+
+void mem_free_p3c(struct mem_list** mem, CHAR**** p, INT a, INT b)
 {
- int i,j;
+ INT i,j;
  debug("%s:%d mem_free_p3c()", __FILE__,__LINE__);
  debug("mem_free_p3c: ****p=%p, a=%d, b=%d\n",(void*)p,a,b);
  if (!(*p)) return ;
@@ -1157,9 +1180,10 @@ void mem_free_p3c(struct mem_list** mem, char**** p, int a, int b)
  sfree(mem, (void**)(&(*p)));
 }
 
-void dmem_free_p3c(struct dmem_list** mem, char**** p, int a, int b, char* file, int line)
+
+void dmem_free_p3c(struct dmem_list** mem, CHAR**** p, INT a, INT b, CHAR* file, INT line)
 {
- int i,j;
+ INT i,j;
  debug("%s:%d dmem_free_p3c() called from %s:%d", __FILE__,__LINE__,file,line);
  debug("dmem_free_p3c: ****p=%p, a=%d, b=%d\n",(void*)p,a,b);
  if (!(*p)) return ;
@@ -1170,9 +1194,10 @@ void dmem_free_p3c(struct dmem_list** mem, char**** p, int a, int b, char* file,
  dsfree(mem, (void**)(&(*p)), HERE);
 }
 
-void mfree_p3c(char**** p, int a, int b)
+
+void mfree_p3c(CHAR**** p, INT a, INT b)
 {
- int i,j;
+ INT i,j;
  debug("%s:%d mfree_p3c()", __FILE__,__LINE__);
  debug("mfree_p3c: ****p=%p, a=%d, b=%d\n",(void*)p,a,b);
  if (!(*p)) return ;
@@ -1183,9 +1208,10 @@ void mfree_p3c(char**** p, int a, int b)
  mfree((void**)(&(*p)));
 }
 
-void dmfree_p3c(char**** p, int a, int b, char* file, int line)
+
+void dmfree_p3c(CHAR**** p, INT a, INT b, CHAR* file, INT line)
 {
- int i,j;
+ INT i,j;
  debug("%s:%d dmfree_p3c() called from: %s:%d", __FILE__,__LINE__,file,line);
  debug("dmfree_p3c: ****p=%p, a=%d, b=%d\n",(void*)p,a,b);
  if (!(*p)) return ;
@@ -1196,9 +1222,10 @@ void dmfree_p3c(char**** p, int a, int b, char* file, int line)
  dmfree((void**)(&(*p)), HERE);
 }
 
-void free_p3c(char**** p, int a, int b)
+
+void free_p3c(CHAR**** p, INT a, INT b)
 {
- int i,j;
+ INT i,j;
  debug("%s:%d free_p3c()", __FILE__,__LINE__);
  debug("free_p3c: ****p=%p, a=%d, b=%d\n",(void*)p,a,b);
  if (!(*p)) return ;
