@@ -9,8 +9,8 @@
 #include "common.h"
 #include "signals.h"
 #include "ilibpq.h"
-extern int ping_conn;
-extern int cont;
+extern INT ping_conn;
+extern INT cont;
 extern PGconn* connection;
 
 void priv_sig_proc()
@@ -29,6 +29,7 @@ void sig_abort()
  cont=0;
 }
 
+
 void discoteka()
 {
  debug("%s:%d discoteka()", __FILE__,__LINE__);
@@ -37,17 +38,19 @@ void discoteka()
  connection = NULL;
 }
 
+
 void sig_int()
 {
  debug("%s:%d sig_kill()", __FILE__,__LINE__);
- cont=0; 
+ cont=0;
  system("killall -9 pbar");
  printf("Exiting by the SIGNAL\n");
  discoteka();
  exit(0);
 }
 
-void alarm_clock(int sec)
+
+void alarm_clock(INT sec)
 {
  debug("%s:%d alarm_clock(%d)", __FILE__,__LINE__,sec);
  alarm(sec);
