@@ -6,10 +6,8 @@
  written in VIM editor, ctags used, CVS used
  Currently only one author: MOrgoth DBMA
  FILE: sqlparser.c */
-
 #include "sqlparser.h"
 #include "common.h"
-
 struct parser_data parser[NUM_PARSE_STRINGS] = {
 { 0,  "ABORT" },
 { 0,  "ABORT TRANSACTION" },
@@ -745,9 +743,9 @@ struct parser_data parser[NUM_PARSE_STRINGS] = {
 { 0,  "ZONE PST8PDT" }
 };
 
-int get_last_words(char* target, char* from, int count)
+INT get_last_words(CHAR* target, CHAR* from, INT count)
 {
- int i,l,offset,cnt;
+ INT i,l,offset,cnt;
  debug("%s:%d get_last_words(%s,%s,%d)", __FILE__,__LINE__,target?target:"null",from?from:"null",count);
  l = strlen(from);
  i = l-1;
@@ -773,23 +771,24 @@ int get_last_words(char* target, char* from, int count)
  return offset;
 }
 
-int auto_complete(char* str, char*** table)
+
+INT auto_complete(CHAR* str, CHAR*** table)
 {
- int count,i,j, len;
+ INT count,i,j, len;
  debug("%s:%d auto_complete(%s, %p)", __FILE__,__LINE__,str?str:"null",table);
  count = 0;
  *table = NULL;
  len = strlen(str);
  for (i=0;i<NUM_PARSE_STRINGS;i++) if (!strncmp(parser[i].keyword,str,len)) count++;
  if (!count) { *table = NULL; return 0; }
- *table = (char**)malloc(count*sizeof(char*));
+ *table = (CHAR**)malloc(count*sizeof(CHAR*));
  if (!(*table)) fatal_here(HERE, "cannot allocate memory.");
  j = 0;
- for (i=0;i<NUM_PARSE_STRINGS;i++) 
+ for (i=0;i<NUM_PARSE_STRINGS;i++)
    {
     if (!strncmp(parser[i].keyword,str,len))
       {
-       (*table)[j] = (char*)malloc((strlen(parser[i].keyword)+1)*sizeof(char));
+       (*table)[j] = (CHAR*)malloc((strlen(parser[i].keyword)+1)*sizeof(CHAR));
        if (!((*table)[j])) fatal_here(HERE, "cannot allocate memory, (I,J)=(%d,%d)",i,j);
        strcpy((*table)[j], parser[i].keyword);
        j++;
