@@ -12,18 +12,14 @@
 #define MAX_QUERY_LENGTH 4096
 #define PARSER_MAX_LENGTH 150
 #define NUM_PARSE_STRINGS 732
-
 #include <string.h>
 #include "consoleout.h"
 #include "mmsystem.h"
-
 struct parser_data
 {
-  int type;
-  char keyword[PARSER_MAX_LENGTH];
+  INT type;
+  CHAR keyword[PARSER_MAX_LENGTH];
 };
-
-int get_last_words(char* target, char* from, int count);
-int auto_complete(char* str, char*** table);
-
+INT get_last_words(CHAR* target, CHAR* from, INT count);
+INT auto_complete(CHAR* str, CHAR*** table);
 #endif
