@@ -655,6 +655,284 @@ void scale_changed_func(GtkWidget* scale, gfloat dummy)
    }
 }
 
+void ctab_prop_combos(GtkWidget** combo, GtkWidget* table, INT I)
+{
+  debug("%s:%d ctab_prop_combos(%d)",__FILE__,__LINE__, I);
+  CHAR*** ptr;
+  INT row, col, i;
+  CHAR errstr[MAX_QUERY_LENGTH];
+  GList* list;
+  CHAR **tmps;
+  if (!want_all_elems)
+  execute_printf(connection, &ptr, &row, &col, errstr, 0,
+	"SELECT tablename from pg_tables WHERE tablename NOT LIKE 'pg_%%' AND tablename NOT LIKE 'sql_%%' ORDER BY tablename");
+  else
+  execute_printf(connection, &ptr, &row, &col, errstr, 0,
+		 "SELECT tablename from pg_tables ORDER BY tablename");
+  if (!ptr) return;
+  if (row<0 || col<0)
+   {
+    gtk_dialog_printf_big("Error executing query:\nReturn from DBMS: %s\n", errstr);
+    return;
+   }
+  list = NULL;
+  list = g_list_append(list, "-other properties-");
+  list = g_list_append(list, "primary key");
+  tmps = (CHAR**)malloc(row*sizeof(CHAR**));
+  for (i=0;i<row;i++)
+    {
+     tmps[i] = (CHAR*)malloc(strlen(ptr[i][0])+16);
+     sprintf(tmps[i], "references %s", ptr[i][0]);
+     list = g_list_append(list, tmps[i]);
+    }
+  *combo = gtk_combo_new();
+  gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(*combo)->entry),"");
+  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(*combo)->entry), FALSE);
+  gtk_combo_set_popdown_strings(GTK_COMBO(*combo), list);
+  g_list_free(list);
+  gtk_table_attach(GTK_TABLE (table), *combo, 16, 24, I+2, I+3,GTK_FILL,GTK_FILL, 0, 0);
+  //gtk_signal_connect(GTK_OBJECT(GTK_COMBO(combo)->entry), "changed", GTK_SIGNAL_FUNC(update_pidata_combo),(gpointer)(i));
+  gtk_widget_show(*combo);
+  for(i=0;i<row;i++) free(tmps[i]);
+  free(tmps);
+  free_p3c(&ptr, row, col);
+}
+
+
+void ctab_type_combos(GtkWidget** combo, GtkWidget* table, INT i)
+{
+ debug("%s:%d ctab_type_combos(%d)",__FILE__,__LINE__, i);
+  GList* list;
+  list = NULL;
+  list = g_list_append(list, "text");
+  list = g_list_append(list, "smallint");
+  list = g_list_append(list, "integer");
+  list = g_list_append(list, "bigint");
+  list = g_list_append(list, "boolean");
+  list = g_list_append(list, "float4");
+  list = g_list_append(list, "float8");
+  list = g_list_append(list, "decimal");
+  list = g_list_append(list, "time");
+  list = g_list_append(list, "timestamp");
+  list = g_list_append(list, "date");
+  list = g_list_append(list, "varchar(1)");
+  list = g_list_append(list, "varchar(2)");
+  list = g_list_append(list, "varchar(4)");
+  list = g_list_append(list, "varchar(10)");
+  list = g_list_append(list, "varchar(20)");
+  list = g_list_append(list, "varchar(40)");
+  list = g_list_append(list, "varchar(100)");
+  list = g_list_append(list, "varchar(255)");
+  list = g_list_append(list, "varchar(1024)");
+  *combo = gtk_combo_new();
+  gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(*combo)->entry),"Select type");
+  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(*combo)->entry), FALSE);
+  gtk_combo_set_popdown_strings(GTK_COMBO(*combo), list);
+  g_list_free(list);
+  gtk_table_attach(GTK_TABLE (table), *combo, 8, 16, i+2, i+3,GTK_FILL,GTK_FILL, 0, 0);
+  //gtk_signal_connect(GTK_OBJECT(GTK_COMBO(combo)->entry), "changed", GTK_SIGNAL_FUNC(update_pidata_combo),(gpointer)(i));
+  gtk_widget_show(*combo);
+}
+
+void update_title(GtkWidget* w, gpointer d)
+{
+ debug("%s:%d update_title()",__FILE__,__LINE__);
+ strcpy((CHAR*)wdata.tabname,gtk_entry_get_text(GTK_ENTRY(w)));
+ debug("tabname=%s\n", wdata.tabname);
+}
+
+void save_ctab(GtkWidget* w, gpointer d);
+
+
+void setup_mainctrls(GtkWidget* table)
+{
+ GtkWidget **name, **type, **props, **anull, **def, *lname, *ltype, *lprops, *lanull, *ldef, *tab, *ltab, *sav;
+ INT i;
+ debug("%s:%d setup_mainctrls()",__FILE__,__LINE__);
+ name  = (GtkWidget**)malloc(wdata.ncols*sizeof(GtkWidget*));
+ if (!name)  fatal_here(HERE, "malloc failed");
+ type  = (GtkWidget**)malloc(wdata.ncols*sizeof(GtkWidget*));
+ if (!type)  fatal_here(HERE, "malloc failed");
+ props = (GtkWidget**)malloc(wdata.ncols*sizeof(GtkWidget*));
+ if (!props) fatal_here(HERE, "malloc failed");
+ anull = (GtkWidget**)malloc(wdata.ncols*sizeof(GtkWidget*));
+ if (!anull) fatal_here(HERE, "malloc failed");
+ def   = (GtkWidget**)malloc(wdata.ncols*sizeof(GtkWidget*));
+ if (!def)   fatal_here(HERE, "malloc failed");
+ sav = gtk_button_new_with_label("Save Query");
+ gtk_table_attach(GTK_TABLE(table), sav, 30, 40, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
+ gtk_widget_show(sav);
+ gtk_signal_connect(GTK_OBJECT(sav), "clicked", GTK_SIGNAL_FUNC(save_ctab), NULL);
+ lname  = gtk_label_new("Column Name:");
+ ltype  = gtk_label_new("Type:");
+ lprops = gtk_label_new("Properties:");
+ lanull = gtk_label_new("NULL control:");
+ ldef   = gtk_label_new("Default value:");
+ ltab   = gtk_label_new("Table name:");
+ gtk_table_attach(GTK_TABLE(table), ltab ,  0,  15, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
+ gtk_table_attach(GTK_TABLE(table), lname , 0,   8, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
+ gtk_table_attach(GTK_TABLE(table), ltype , 8,  16, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
+ gtk_table_attach(GTK_TABLE(table), lprops, 16, 24, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
+ gtk_table_attach(GTK_TABLE(table), lanull, 24, 32, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
+ gtk_table_attach(GTK_TABLE(table), ldef  , 32, 40, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
+ gtk_widget_show(lname);
+ gtk_widget_show(ltype);
+ gtk_widget_show(lprops);
+ gtk_widget_show(lanull);
+ gtk_widget_show(ldef);
+ gtk_widget_show(ltab);
+ tab = gtk_entry_new();
+ gtk_entry_set_max_length(GTK_ENTRY(tab), STDSTRING-1);
+ gtk_entry_set_editable(GTK_ENTRY(tab), TRUE);
+ gtk_table_attach(GTK_TABLE(table), tab  , 15, 30, 0, 1, GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL, 0, 0);
+ gtk_signal_connect(GTK_OBJECT(tab), "activate", GTK_SIGNAL_FUNC(update_title), NULL);
+ gtk_signal_connect(GTK_OBJECT(tab), "changed",  GTK_SIGNAL_FUNC(update_title), NULL);
+ gtk_widget_show(tab);
+ for (i=0;i<wdata.ncols;i++)
+  {
+   name[i] = gtk_entry_new();
+   gtk_entry_set_max_length(GTK_ENTRY(name[i]), 127);
+   gtk_entry_set_editable(GTK_ENTRY(name[i]), TRUE);
+   //gtk_signal_connect(GTK_OBJECT(name[i]), "activate", GTK_SIGNAL_FUNC(update_entry), NULL);
+   //gtk_signal_connect(GTK_OBJECT(name[i]), "changed",  GTK_SIGNAL_FUNC(update_entry), NULL);
+   gtk_table_attach(GTK_TABLE(table), name[i], 0, 8, i+2, i+3, GTK_FILL|GTK_EXPAND|GTK_SHRINK, GTK_FILL, 0, 0);
+   gtk_widget_show(name[i]);
+   ctab_type_combos(&type[i], table, i);
+   ctab_prop_combos(&props[i], table, i);
+   anull[i] = gtk_check_button_new_with_label("Allow NULL");
+   //gtk_signal_connect(GTK_OBJECT(anull[i]), "toggled",GTK_SIGNAL_FUNC(entry_toggle_editable), entry);
+   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(anull[i]), FALSE);
+   gtk_table_attach(GTK_TABLE(table), anull[i], 24, 32, i+2, i+3, GTK_FILL, GTK_FILL, 0, 0);
+   gtk_widget_show(anull[i]);
+   def[i] = gtk_entry_new();
+   gtk_entry_set_max_length(GTK_ENTRY(def[i]), STDSTRING);
+   gtk_entry_set_editable(GTK_ENTRY(def[i]), TRUE);
+   //gtk_signal_connect(GTK_OBJECT(def[i]), "activate", GTK_SIGNAL_FUNC(update_entry), NULL);
+   //gtk_signal_connect(GTK_OBJECT(def[i]), "changed",  GTK_SIGNAL_FUNC(update_entry), NULL);
+   gtk_table_attach(GTK_TABLE(table), def[i], 32, 40, i+2, i+3, GTK_FILL, GTK_FILL, 0, 0);
+   gtk_widget_show(def[i]);
+  }
+ wdata.tab    = tab;
+ wdata.sav    = sav;
+ wdata.ltab   = ltab;
+ wdata.name   = name;
+ wdata.type   = type;
+ wdata.props  = props;
+ wdata.anull  = anull;
+ wdata.def    = def;
+ wdata.lname  = lname;
+ wdata.ltype  = ltype;
+ wdata.lprops = lprops;
+ wdata.lanull = lanull;
+ wdata.ldef   = ldef;
+}
+
+int try_exec(char* errptr)
+{
+  CHAR*** ptr;
+  INT row, col;
+  debug("%s:%d try_exec()",__FILE__,__LINE__);
+  if (!qptr) { sprintf(errptr, "Query is NULL"); return 0; }
+  execute_printf(connection, &ptr, &row, &col, errptr, 0, qptr);
+  if (row<0 || col<0) return 0; //if !ptr then ...
+  free_p3c(&ptr, row, col);
+  
+  execute_printf(connection, &ptr, &row, &col, errptr, 0, "DROP TABLE %s", wdata.tabname);
+  if (row<0 || col<0) return 0; //if !ptr then ...
+  free_p3c(&ptr, row, col);
+  return 1;
+}
+
+void generate_query_ctab()
+{
+ INT i;
+ char tmps[STDSTRING+64];
+ char tmps2[STDSTRING+64];
+ debug("%s:%d generate_query_ctab()",__FILE__,__LINE__);
+ if (qptr) free(qptr);
+ qptr = NULL;
+ qptr = malloc(wdata.ncols*(STDSTRING+64)+MAX_QUERY_LENGTH);
+ if (!qptr) fatal_here(HERE, "malloc failed!");
+ if (!strcmp(wdata.tabname, "")) 
+   {
+    gtk_dialog_printf("Set table name!");
+    free(qptr);
+    qptr = NULL;
+    return ;
+   }
+ sprintf(qptr, "CREATE TABLE %s(", wdata.tabname);
+ for (i=0;i<wdata.ncols;i++)
+   {
+    sprintf(tmps, "%s %s", gtk_entry_get_text(GTK_ENTRY(wdata.name[i])), 
+	    gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wdata.type[i])->entry)));
+    if (!(GTK_TOGGLE_BUTTON(wdata.anull[i])->active))
+      {
+       strcat(tmps, " NOT NULL");
+      }
+    if (strcmp(gtk_entry_get_text(GTK_ENTRY(wdata.def[i])),""))
+      {
+       if (numeric(gtk_entry_get_text(GTK_ENTRY(wdata.def[i]))))
+       sprintf(tmps2," DEFAULT %s", gtk_entry_get_text(GTK_ENTRY(wdata.def[i])));
+       else 
+       sprintf(tmps2," DEFAULT '%s'", gtk_entry_get_text(GTK_ENTRY(wdata.def[i])));
+       strcat(tmps, tmps2);
+      }
+    if (strcmp(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wdata.props[i])->entry)), "-other properties-"))
+      {
+       if (!strcmp(gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wdata.props[i])->entry)), "primary key"))
+	   strcat(tmps, " PRIMARY KEY");
+       else
+         {
+          sprintf(tmps2, " %s", gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(wdata.props[i])->entry)));
+	  strcat(tmps, tmps2);
+         }
+      }
+    if (i < (wdata.ncols-1)) strcat(tmps,", ");
+    else strcat(tmps, ")");
+    strcat(qptr, tmps);
+   }
+ debug("qptr = %s\n", qptr);
+}
+
+void save_ctab(GtkWidget* w, gpointer d)
+{
+ debug("%s:%d save_ctab()",__FILE__,__LINE__);
+ generate_query_ctab();
+ CHAR *filen;
+ CHAR filename[MAXPATH];
+ FILE* f;
+ INT l;
+ l = strlen(qptr);
+ if (l<3)
+    {
+     gtk_dialog_printf("Will not save empty query.");
+     return;
+    }
+ filen = gtk_dialog_getfile(".QUE", "Enter filename to save query");
+ if (!strcmp(filen,""))
+   {
+    gtk_dialog_printf("Cannot save query to non-QUE file\n");
+    return;
+   }
+ strcpy(filename,filen);
+ f = fopen(filename,"r");
+ if (f)
+   {
+    fclose(f);
+    if (!gtk_dialog_yes_no_printf("Overwite file: %s?", filename)) return;
+   }
+ f = fopen(filename,"w");
+ if (!f)
+   {
+    gtk_dialog_printf("Cannot write to: %s\n", filename);
+    return ;
+   }
+ fprintf(f,"%s", qptr);
+ fclose(f);
+ if (w) gtk_dialog_printf("Saved to %s\n", filename);
+ debug("qptr=%s\n", qptr);
+}
+
 
 void setup_txtscene(GtkWidget* table)
 {
@@ -730,6 +1008,31 @@ void prev_ctable_func(GtkWidget* w, gpointer data)
                gtk_widget_destroy(wdata.hscrl);
                gtk_widget_destroy(wdata.vscrl);
 	      }
+	    else
+	      {
+	       INT x;
+               gtk_widget_destroy(wdata.sav);
+               gtk_widget_destroy(wdata.tab);
+               gtk_widget_destroy(wdata.ltab);
+               gtk_widget_destroy(wdata.lname);
+               gtk_widget_destroy(wdata.ltype);
+               gtk_widget_destroy(wdata.lprops);
+               gtk_widget_destroy(wdata.lanull);
+               gtk_widget_destroy(wdata.ldef);
+	       for (x=0;x<wdata.ncols;x++)
+	         {
+                  gtk_widget_destroy(wdata.name[x]);
+                  gtk_widget_destroy(wdata.type[x]);
+                  gtk_widget_destroy(wdata.props[x]);
+                  gtk_widget_destroy(wdata.anull[x]);
+                  gtk_widget_destroy(wdata.def[x]);
+	         }
+	       free(wdata.name);
+	       free(wdata.type);
+	       free(wdata.props);
+	       free(wdata.anull);
+	       free(wdata.def);
+	      }
  	    adj = gtk_adjustment_new(0.0,0.0,49.0, 1.0, 1.0, 1.0);
 	    scale = gtk_hscale_new(GTK_ADJUSTMENT(adj));
 	    gtk_scale_set_digits(GTK_SCALE(scale), 0);
@@ -756,7 +1059,6 @@ void prev_ctable_func(GtkWidget* w, gpointer data)
  }
 }
 
-
 void next_ctable_func(GtkWidget* w, gpointer data)
 {
  debug("%s:%d next_ctable_func()", __FILE__,__LINE__);
@@ -797,6 +1099,8 @@ void next_ctable_func(GtkWidget* w, gpointer data)
 	       gtk_widget_show(tab);
 	       break;
 	      }
+	    ieditor = 0;
+	    setup_mainctrls(tab);
 	    gtk_widget_show(tab);
 	    wizstep++;
 	    break;
@@ -844,9 +1148,42 @@ void next_ctable_func(GtkWidget* w, gpointer data)
 	         gtk_main_quit();
 		 break;
 	      }
-	    /* HERE CASE WHEN USER WANTS TO CREATE COLUMNS */
-	    gtk_dialog_printf("Not implemented!");
-	    qptr = NULL;
+	    INT x;
+	    CHAR ans[MAX_QUERY_LENGTH*20+100];
+	    generate_query_ctab();
+	    if (!try_exec(ans))
+	      {
+		ans[191] = 0;		//may be too long!
+  		if (gtk_dialog_yes_no_printf_big("DB error:\n%s\nClose wizard?", ans))
+		  {
+		   free(qptr);
+   	           qptr = NULL;
+	           gtk_main_quit();
+		   break;
+		  }
+		else { free(qptr); qptr = NULL; break; }
+	      }
+            gtk_widget_destroy(wdata.sav);
+            gtk_widget_destroy(wdata.tab);
+            gtk_widget_destroy(wdata.ltab);
+            gtk_widget_destroy(wdata.lname);
+            gtk_widget_destroy(wdata.ltype);
+            gtk_widget_destroy(wdata.lprops);
+            gtk_widget_destroy(wdata.lanull);
+            gtk_widget_destroy(wdata.ldef);
+	    for (x=0;x<wdata.ncols;x++)
+	         {
+                  gtk_widget_destroy(wdata.name[x]);
+                  gtk_widget_destroy(wdata.type[x]);
+                  gtk_widget_destroy(wdata.props[x]);
+                  gtk_widget_destroy(wdata.anull[x]);
+                  gtk_widget_destroy(wdata.def[x]);
+	         }
+	    free(wdata.name);
+	    free(wdata.type);
+	    free(wdata.props);
+	    free(wdata.anull);
+	    free(wdata.def);
 	    gtk_main_quit();
 	    break;
     default:	/* other - finish? */
@@ -858,26 +1195,33 @@ void next_ctable_func(GtkWidget* w, gpointer data)
 }
 
 
+
 void init_wdata()
 {
  debug("%s:%d init_wdata()", __FILE__,__LINE__);
  ieditor = 0;
  wdata.ncols = 4;
- wdata.label = wdata.label2 = wdata.scale = wdata.table =
- wdata.text  = wdata.load   = wdata.save  = wdata.hscrl = wdata.vscrl = NULL;
+ wdata.label = wdata.label2 = wdata.scale  = wdata.table =
+ wdata.text  = wdata.load   = wdata.save   = wdata.hscrl = wdata.vscrl =
+ wdata.lname = wdata.ltype  = wdata.lanull = wdata.ldef  = NULL;
+ wdata.name  = wdata.type   = wdata.anull  = wdata.def   = NULL;
+ wdata.tab   = wdata.ltab   = wdata.sav    = NULL;
 }
 
 
+
 CHAR* ctable_wizard()
 {
  debug("%s:%d ctable_wizard()", __FILE__,__LINE__);
+ //FIXME if freed but not NULLed ?? Fucked up things..
+ //I hope I removed that possibilities
  if (qptr) free(qptr);
  qptr = NULL;
  wizstep = 0;
  init_wdata();
  GtkWidget* window, *table, *button_next, *button_prev, *scroller_view, *vbox, *hbox, *label;
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
- gtk_widget_set_usize(GTK_WIDGET(window), 500, 400);
+ gtk_widget_set_usize(GTK_WIDGET(window), 780, 450);
  gtk_container_set_border_width(GTK_CONTAINER(window), 5);
  table = gtk_table_new(50, 50, FALSE);
  label = gtk_label_new("Welcome to Create Table Wizard\nthis wizard will guide You\nthrough the rest of Create Table process\nGood luck!");
