@@ -13,6 +13,7 @@
 #ifndef __MORGOTHDBMA_WIZARDS_H__
 #define __MORGOTHDBMA_WIZARDS_H__
 #include "main.h"
+#define STDSTRING 1024
 #define DEL_TYPE_EXACTLY	0
 #define DEL_TYPE_BEGINS		1
 #define DEL_TYPE_ENDS		2
@@ -38,7 +39,21 @@ struct WizData
  GtkWidget* vscrl;
  GtkWidget* hscrl;
  GtkWidget* text;
+ GtkWidget* lname;
+ GtkWidget* ltype;
+ GtkWidget* lprops;
+ GtkWidget* lanull;
+ GtkWidget* ldef;
+ GtkWidget* ltab;
+ GtkWidget* tab;
+ GtkWidget* sav;
+ GtkWidget** name;
+ GtkWidget** type;
+ GtkWidget** props;
+ GtkWidget** anull;
+ GtkWidget** def;
  INT ncols;
+ CHAR tabname[STDSTRING+1];
 };
 CHAR* insert_wizard(CHAR* tabname);
 CHAR* delete_wizard(CHAR* tabname);
